@@ -219,19 +219,47 @@ public class NLink_v5 {
 		}
 	}
 
-	public void doCommandWhile(final String s, final String cond) throws LogoException, CompilerException
+	public void doCommandWhile(final String s, final String cond, Integer maxMinutes) throws LogoException, CompilerException
 	{
-		while (((Boolean)workspace.report(cond)).booleanValue())
-		{
-			workspace.command(s);
+		if (maxMinutes > 0) {
+			long startTime = System.currentTimeMillis();
+			while (((Boolean)workspace.report(cond)).booleanValue())
+			{
+				workspace.command(s);
+				// max. time exceeded
+				if ((System.currentTimeMillis() - startTime) / 60000 >= maxMinutes) {
+					//break;
+					throw new RuntimeException("Maximum time for NLDoCommandWhile reached. Process stopped.");
+				}
+			}
+		}
+		else {		
+			while (((Boolean)workspace.report(cond)).booleanValue())
+			{
+				workspace.command(s);
+			}
 		}
 	}
 
-	public void doCommandWhile(final String[] s, final String cond) throws LogoException, CompilerException
+	public void doCommandWhile(final String[] s, final String cond, Integer maxMinutes) throws LogoException, CompilerException
 	{
-		while (((Boolean)workspace.report(cond)).booleanValue())
-		{
-			command(s);
+		if (maxMinutes > 0) {
+			long startTime = System.currentTimeMillis();
+			while (((Boolean)workspace.report(cond)).booleanValue())
+			{
+				command(s);
+				// max. time exceeded
+				if ((System.currentTimeMillis() - startTime) / 60000 >= maxMinutes) {
+					//break;
+					throw new RuntimeException("Maximum time for NLDoCommandWhile reached. Process stopped.");
+				}
+			}
+		}
+		else {		
+			while (((Boolean)workspace.report(cond)).booleanValue())
+			{
+				command(s);
+			}
 		}
 	}
 
@@ -292,13 +320,27 @@ public class NLink_v5 {
 
 
 /* Repeats a command and returns a reporter until a condition is met */
-	public Object[] doReportWhile(final String s, final String var, final String condition)
+	public Object[] doReportWhile(final String s, final String var, final String condition, Integer maxMinutes)
 		throws LogoException, CompilerException, Exception
 	{
 		java.util.ArrayList<Object> varList = new java.util.ArrayList<Object>();
-		for(int i=0; ((Boolean)workspace.report(condition)).booleanValue(); i++) {
-			workspace.command(s);
-			varList.add(report(var));
+		if (maxMinutes > 0) {
+			long startTime = System.currentTimeMillis();
+			for(int i=0; ((Boolean)workspace.report(condition)).booleanValue(); i++) {
+				workspace.command(s);
+				varList.add(report(var));
+				// max. time exceeded
+				if ((System.currentTimeMillis() - startTime) / 60000 >= maxMinutes) {
+					//break;
+					throw new RuntimeException("Maximum time for NLDoReportWhile reached. Process stopped.");
+				}
+			}
+		}
+		else {		
+			for(int i=0; ((Boolean)workspace.report(condition)).booleanValue(); i++) {
+				workspace.command(s);
+				varList.add(report(var));
+			}
 		}
 		Object[] objArray = varList.toArray();
 		return objArray;
@@ -306,13 +348,27 @@ public class NLink_v5 {
 
 	
 /* Repeats a command and returns a list of reporters until a condition is met */
-	public Object[] doReportWhile(final String s, final String[] vars, final String condition)
+	public Object[] doReportWhile(final String s, final String[] vars, final String condition, Integer maxMinutes)
 		throws LogoException, CompilerException, Exception
 	{
 		java.util.ArrayList<Object> varList = new java.util.ArrayList<Object>();
-		for(int i=0; ((Boolean)workspace.report(condition)).booleanValue(); i++) {
-			workspace.command(s);
-			varList.add(report(vars));
+		if (maxMinutes > 0) {
+			long startTime = System.currentTimeMillis();
+			for(int i=0; ((Boolean)workspace.report(condition)).booleanValue(); i++) {
+				workspace.command(s);
+				varList.add(report(vars));
+				// max. time exceeded
+				if ((System.currentTimeMillis() - startTime) / 60000 >= maxMinutes) {
+					//break;
+					throw new RuntimeException("Maximum time for NLDoReportWhile reached. Process stopped.");
+				}
+			}
+		}
+		else {		
+			for(int i=0; ((Boolean)workspace.report(condition)).booleanValue(); i++) {
+				workspace.command(s);
+				varList.add(report(vars));
+			}
 		}
 		Object[] objArray = varList.toArray();
 		return objArray;
